#include "LocalVField.h"

LocalVField::LocalVField(IMap F_, IVector fixedPoint_,IMatrix C_, IMap psi_)
{
	F=F_;
	fixedPoint=fixedPoint_;
	C=C_;
    psi=psi_;
    
    F.setDegree(2);
    psi.setDegree(2);
}

IVector LocalVField::Psi(const IVector &p) const
{
    return fixedPoint+C*psi(p);
}

IMatrix LocalVField::DPsi(const IVector &p) const
{
    return C*psi[p];
}

IVector LocalVField::image(const IVector &p) const
{
    return gauss(DPsi(p), F(Psi(p)) );
}

IVector LocalVField::operator()(const IVector &p) const
{
    return image(p);
}

//////////////////////////
// This function computes the matrix D2psi(q)*Floc(q)
// this is needed later for the computation of the derivative of the
// local vector field. (see function ThreeBPVectorField::DFloc)
IMatrix LocalVField::D2psi(const IVector &q) const
{
    int n = q.dimension();
    
    IMatrix D(n,n);
    IHessian D2(n,n);
    
    IMatrix res(n,n);
    
    psi(q,D,D2);
    
    IVector Flocq = image(q);
    for (int i=0; i<n; i++)
    {
        for(int k=0; k<n; k++)
        {
            for (int j = 0; j < n; j++)
            {
                if(k==j) D2(i, k, j) = 2.0*D2(i, k, j);
                res[i][k] = res[i][k] + D2(i, k, j) * Flocq[j];
            }
        }
    }
    return res;
}

// This is the function which computes DFloc(p)
IMatrix LocalVField::derivative(const IVector &p) const
{
    return gaussInverseMatrix(psi[p]) * ( gaussInverseMatrix(C)*F[Psi(p)]*DPsi(p) - D2psi(p) );
}
//////////////////////////


IMatrix LocalVField::operator[](const IVector &p) const
{
 return derivative(p);
}

////////////////////////////////////////////////////
// Function: set_init_cond()
//
// Purpose: The function transforms the set Bloc which is in local coordinates to original coordiantes
// Remark: This is done for using the set in original coordinates to initialize Poincare Map, thus
// we need to enclose set B in proper form as a type "C0Rect2Set"
// Note: C0Rect2Set represent a subset of R^n in the form of doubleton
// x+C*r_0+Q*q,
// where
// x,q,r0 are interval vectors, where x is a point interval vector
// C,Q are interval matrices, with Q close to orthogonal

C0HORect2Set LocalVField::set_init_cond(const IVector &Bloc) const
{    
    IVector x_0 = fixedPoint+C*psi( midVector(Bloc));

    C0HORect2Set B_init(x_0,DPsi(Bloc),Bloc-midVector(Bloc));
    
    
    return B_init;
}