#ifndef conecond_h
#define conecond_h

#include <iostream>

#include "capd/intervals/lib.h"
#include "capd/vectalg/lib.h"

#include "utils.h"

using namespace capd; 

////////////////////
// Function: satisfiesConeConditions (where arguments are two matrices A an B)
//
// The purpose of this function is to check if for sufficiently small t>0 the map
// phi_t satisfies cone conditions for (Q,m), where
// Q(x,y)=alpha^2||x||^2-||y||^2
// m=1+2t*ni,
// if we know that
// [DF(U)] is a subset of an interval matrix which has the form
// A  E1
// E2 B,
// such that the euclidean norms of E1 and E2 are e1 and e2.
bool satisfiesConeConditions(const interval &alpha,	const IMatrix &A,const IMatrix &B, const interval &e1,const interval &e2,const interval &ni);

////////////////////
// Function: satisfiesConeConditions (where arguments is the matrix DF)
//
// The purpose of this function is to check if for sufficiently small t>0 the map
// phi_t satisfies cone conditions for (Q,m), if we are given [DF(U)], where
// Q(x,y)=alpha^2||x||^2-||y||^2
// m=1+2t*ni							
bool satisfiesConeConditions(const interval &alpha, int u, const IMatrix &DF, const interval &ni);

#endif
