#include "utils.h"

void abort(string text)
{
	cout << text << endl;
	interval zero=0.0;
	interval error=1.0/zero; // illegal operation to force CAPD to stop computations.
}

IMatrix IdMatrix(int n)
{
    IMatrix Id(n,n);
    for(int i=0;i<n;i++) Id[i][i] = 1.0;
    return Id;
}

void intervalHull(const IMatrix &A, IMatrix &B)
{
    int n = A.numberOfRows();
    int m = A.numberOfColumns();
    if ( ! ( (B.numberOfRows()==n) && (B.numberOfColumns()==m) ) ) abort("Unable to compute the intervall hull of matrices - problems with dimensions. ABORTING program. ");
    for(int i=0;i<n;i++)
    {
        for(int j=0;j<m;j++) B[i][j]=intervalHull(B[i][j],A[i][j]);
    }
}


IVector FixedPoint(IMap &f, const IVector &initialGuess)
{
    IVector X=initialGuess;
    
    int n=initialGuess.dimension();
    
    for(int i=0;i<1000;i++) X=midVector(X)-gauss(f[X],f(midVector(X)));
    
    IVector b(n);
    for(int i=0;i<n;i++)
    {
        b[i]=pow(10.0,-15.0)*interval(-1.0,1.0);
        if(abs(X[i]-X[i].mid())>abs(b[i])) b[i] = 1.1*X[i]-X[i].mid();
    }
    
    IVector x = X + b;
    IVector x_0 = midVector(x);
    
    IVector N = x_0 - gauss(f[x],f(x_0));
    if(!(subsetInterior(N,x))) abort("unable to verify the enclosure of the fixed point. ABORTING program. ");
    return x;
    
}

