#ifndef utils_h
#define utils_h

#include <iostream>

#include "capd/capdlib.h"

using namespace std;
using namespace capd;
using namespace capd::matrixAlgorithms;

///////////////
// Function: part()
//
// This function disects an interval into 'N' parts and returns the k-th fragment, where
// k is a number from {0,1,...,N-1}
inline interval part(const interval &x,const int &N, const int &k)
{
    return x.left()+k*(x.right()-x.left())/N+(x-x.left())/N;
}

///////////////
// Function: abort()
//
// We use this function to stop our computations. This is computed in particular when some estimates do 
// not go through and the program needs to be terminated. While terminating the program we can write out 
// a text report on what went wrong and which estimates did not go through.
void abort(string text);


////////////////
// Function: IdMatrix()
//
// This function returns the identity matrix of dimension n x n.
IMatrix IdMatrix(int n);

////////////////
// Function: intervallHull()
//
// This function produces the intervall hull of two matrices.
void intervalHull(const IMatrix &A, IMatrix &B);

//////////////////////////
// Function: fixedPoint()
//
// This function finds the rigourous enclosure of an equilibrium point for
// an ODE \dot{X}=f(X).
// The enclosure is found using Newton method.
//
// f - vector field
// initialGuess - the intial guess point assumed to be close to the equilibrium point
IVector FixedPoint(IMap &f, const IVector &initialGuess);

#endif
